package edu.uoc.petfinder.controller;

import edu.uoc.petfinder.dto.LocationDto;
import edu.uoc.petfinder.dto.PetDto;
import edu.uoc.petfinder.dto.SimplePetDto;
import edu.uoc.petfinder.dto.UserDto;
import edu.uoc.petfinder.model.Pet;
import edu.uoc.petfinder.spatial.SpatialService;
import edu.uoc.petfinder.spatial.impl.SpatialOperations;
import edu.uoc.petfinder.spatial.impl.SpatialRepository;
import edu.uoc.petfinder.service.ImageService;
import edu.uoc.petfinder.service.PetService;
import edu.uoc.petfinder.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.NoArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.geo.Distance;
import org.springframework.data.geo.GeoPage;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.util.UriComponentsBuilder;

import javax.validation.Valid;
import java.net.URI;

@Api(description = "Pet Controller for pet operations")
@RestController
@RequestMapping("/pet")
@AllArgsConstructor
public class PetController {

    private PetService petService;

    private UserService userService;

    private ImageService imageService;

    private SpatialService spatialService;

    @ApiOperation(value = "Create new pets")
    @PreAuthorize("isAuthenticated()")
    @PostMapping
    public ResponseEntity savePet(@Valid @RequestBody PetDto petDto, @RequestAttribute(name = "user") UserDto userDto, UriComponentsBuilder builder){
        PetDto createdPet = this.petService.createPet(petDto, userDto.getEmail());
        URI location = builder.replacePath("/pet/{id}").buildAndExpand(createdPet.getId()).toUri();
        return ResponseEntity.created(location).body(createdPet);
    }

    @GetMapping("/found")
    public ResponseEntity getFoundPetsByLocation(Pageable pageable, LocationDto location) {
        ResponseEntity responseEntity;
        if (location == null || location.isEmpty()) {
            Page<SimplePetDto> petDtoPage = this.petService.findFoundPetsOrderedByDateDesc(pageable);
            responseEntity = ResponseEntity.ok(petDtoPage);
        } else {
            GeoPage<SimplePetDto> pets = this.spatialService.searchFoundSimplePetByLocation(location, pageable);
            responseEntity = ResponseEntity.ok(pets);
        }
        return responseEntity;
    }

    @ApiOperation(value = "Delete one pet by its id")
    @PreAuthorize("isAuthenticated()")
    @DeleteMapping("/{petId}")
    public ResponseEntity deletePetById(@PathVariable String petId, @RequestAttribute(name = "user") UserDto userDto) {
        this.petService.deletePet(petId, Boolean.TRUE, userDto);
        return ResponseEntity.noContent().build();
    }

    @ApiOperation(value = "Get one pet by its id")
    @GetMapping("/{petId}")
    public ResponseEntity getPetById(@PathVariable String petId) {
            PetDto petDto = this.petService.getPetById(petId);
        return ResponseEntity.ok(petDto);
    }

    @PreAuthorize("isAuthenticated()")
    @PostMapping("/{petId}/images")
    public ResponseEntity savePetImages(@PathVariable String petId, @RequestParam(name = "images") MultipartFile[] images, @RequestAttribute(name = "user") UserDto userDto) {
        PetDto petDto = this.petService.getPetByIdAndOwner(petId, userDto);
        petDto = this.imageService.addImagesToPetDto(images, petDto);
        PetDto savedPetDto = this.petService.updatePet(petDto, userDto.getEmail());
        return ResponseEntity.ok(savedPetDto);
    }

    @PreAuthorize("isAuthenticated()")
    @DeleteMapping("/{petId}/images/{imageId}")
    public ResponseEntity deleteImageFromPet(@PathVariable String petId, @PathVariable String imageId, @RequestAttribute(name = "user") UserDto userDto) {
        PetDto petDto = this.petService.getPetByIdAndOwner(petId, userDto);
        petDto = this.imageService.removeImageFromPetDto(imageId, petDto);
        PetDto savedPetDto = this.petService.updatePet(petDto, userDto.getEmail());
        return ResponseEntity.ok(savedPetDto);
    }

    @PreAuthorize("isAuthenticated()")
    @PutMapping("/{petId}")
    public ResponseEntity updatePet(@PathVariable String petId, @Valid @RequestBody PetDto updatePetDto, @RequestAttribute(name = "user") UserDto userDto) {
        PetDto petDto = this.petService.getPetByIdAndOwner(petId, userDto);
        petDto = this.petService.mergeAndUpdatePet(petDto, updatePetDto, userDto);
        return ResponseEntity.ok(petDto);
    }

    @GetMapping
    public ResponseEntity getPetsByLocation(Pageable pageable, LocationDto location) {
        ResponseEntity responseEntity = null;
        if (location == null || location.isEmpty()) {
            Page<SimplePetDto> petDtoPage = this.petService.findAllPetsOrderedByDateDesc(pageable);
            responseEntity = ResponseEntity.ok(petDtoPage);
        } else {
            GeoPage<SimplePetDto> pets = this.spatialService.searchSimplePetByLocation(location, pageable);
            responseEntity = ResponseEntity.ok(pets);
        }
        return responseEntity;
    }

}
