package edu.uoc.petfinder.initialization;

import edu.uoc.petfinder.constant.RoleType;
import edu.uoc.petfinder.model.Action;
import edu.uoc.petfinder.model.Pet;
import edu.uoc.petfinder.model.Role;
import edu.uoc.petfinder.model.User;
import edu.uoc.petfinder.repository.RoleRepository;
import edu.uoc.petfinder.repository.UserRepository;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.bson.BsonDocument;
import org.bson.BsonString;
import org.bson.conversions.Bson;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.index.GeoSpatialIndexType;
import org.springframework.data.mongodb.core.index.GeospatialIndex;
import org.springframework.data.mongodb.core.index.MongoPersistentEntityIndexResolver;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;

import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.Month;
import java.util.Collections;

@Component
@AllArgsConstructor
@Slf4j
public class DataInitialization implements ApplicationListener<ContextRefreshedEvent> {

    private RoleRepository roleRepository;

    private UserRepository userRepository;

    private PasswordEncoder encoder;

    private MongoTemplate mongoTemplate;

    @Override
    public void onApplicationEvent(ContextRefreshedEvent contextRefreshedEvent) {
        this.populateRoles();
        this.createAdminUser();
        this.createIndexes();
    }

    private void createAdminUser() {
        Role adminRole = this.roleRepository.findByName(RoleType.ROLE_ADMIN).get();
        if (!this.userRepository.existsByRoles(adminRole)) {
            User user = new User();
            user.setEmail("admin@ricard.es");
            user.setPassword(this.encoder.encode("Pass123."));
            user.setFirstName("Ricard");
            user.setLastName("Molina Ferret");
            user.setIsSocial(Boolean.FALSE);
            user.setBirthDate(ZonedDateTime.of(1991, 12, 23, 0, 0, 0, 0, ZoneId.systemDefault()));
            user.setRoles(Collections.singleton(adminRole));
            this.userRepository.save(user);
            log.info("INIT -- ADMIN USER CREATED OK");
        }
    }

    private void populateRoles() {
        if (this.roleRepository.count() == 0L) {
            Role userRole = new Role();
            userRole.setName(RoleType.ROLE_USER);
            Role adminRole = new Role();
            adminRole.setName(RoleType.ROLE_ADMIN);
            this.roleRepository.save(userRole);
            this.roleRepository.save(adminRole);
            log.info("INIT -- ROLES POPULATED OK");
        }
    }

    private void createIndexes() {
        this.mongoTemplate.indexOps(Pet.class).ensureIndex(new GeospatialIndex("location.coordinates").typed(GeoSpatialIndexType.GEO_2DSPHERE).named("pets2DSphereIndex"));
        this.mongoTemplate.indexOps(Action.class).ensureIndex(new GeospatialIndex("location.coordinates").typed(GeoSpatialIndexType.GEO_2DSPHERE).named("actions2DSphereIndex"));
        log.info("INIT -- CREATION INDEXES");
    }
}
