package edu.uoc.petfinder.security;

import com.google.api.client.googleapis.auth.oauth2.GoogleIdToken;
import com.google.api.client.googleapis.auth.oauth2.GoogleIdTokenVerifier;
import com.google.api.client.http.javanet.NetHttpTransport;
import com.google.api.client.json.jackson2.JacksonFactory;
import edu.uoc.petfinder.constant.RoleType;
import edu.uoc.petfinder.dto.LoginResponseDto;
import edu.uoc.petfinder.dto.RoleDto;
import edu.uoc.petfinder.dto.TokenDto;
import edu.uoc.petfinder.dto.UserDto;
import edu.uoc.petfinder.exception.PetFinderException;
import edu.uoc.petfinder.mail.EmailFactory;
import edu.uoc.petfinder.mapper.ApiMapper;
import edu.uoc.petfinder.repository.RoleRepository;
import edu.uoc.petfinder.repository.UserRepository;
import edu.uoc.petfinder.service.UserService;

import lombok.AllArgsConstructor;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.social.facebook.api.Facebook;
import org.springframework.social.facebook.api.User;
import org.springframework.social.facebook.api.impl.FacebookTemplate;
import org.springframework.stereotype.Service;

import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.Set;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class AuthenticationService {

    private final AuthenticationManager authenticationManager;

    private final UserRepository userRepository;

    private final PasswordEncoder encoder;

    private final JwtTokenService tokenService;

    private final UserService userService;

    private final ApiMapper mapper;

    @Value("${petfinder.google.id}")
    private String googleId;

    @Value("${petfinder.social.password}")
    private String defaultSocialPassword;

    public LoginResponseDto signIn(UserDto userDto) {
        Authentication authentication;
        try {
            authentication = authenticationManager.authenticate(
                    new UsernamePasswordAuthenticationToken(userDto.getEmail(), userDto.getPassword()));
        } catch (Exception ex) {
            throw new PetFinderException("Error: Unauthorized", HttpStatus.UNAUTHORIZED);
        }

        SecurityContextHolder.getContext().setAuthentication(authentication);
        return prepareLoginResponse(authentication);
    }

    private LoginResponseDto prepareLoginResponse(Authentication authentication) {
        String jwt = tokenService.generateToken(authentication);

        UserDto userDto = this.userService.getUserByEmail(authentication.getName());

        return LoginResponseDto.builder()
                .firstName(userDto.getFirstName())
                .lastName(userDto.getLastName())
                .email(userDto.getEmail())
                .roles(userDto.getRoles())
                .token(jwt)
                .build();
    }

    public LoginResponseDto signUp(UserDto userDto) {
        userDto.setIsSocial(Boolean.FALSE);
        UserDto createdUser = this.userService.createUser(userDto);
        return this.autoLoginUser(createdUser);
    }

    private LoginResponseDto autoLoginUser(UserDto createdUser) {
        UserDetailsImpl user = UserDetailsImpl.build(this.mapper.userDtoToUser(createdUser));
        Authentication authentication = new UsernamePasswordAuthenticationToken(user, null, user.getAuthorities());
        SecurityContextHolder.getContext().setAuthentication(authentication);
        return this.prepareLoginResponse(authentication);
    }

    public LoginResponseDto connectWithFacebook(TokenDto tokenDto) {
        UserDto userDto = this.extractFacebookUserFromToken(tokenDto);

        if (!this.userRepository.existsByEmail(userDto.getEmail())) {
            userDto.setIsSocial(Boolean.TRUE);
            userDto = this.userService.createUser(userDto);
        } else {
            userDto = this.userService.getUserByEmail(userDto.getEmail());
        }

        return this.autoLoginUser(userDto);
    }

    private UserDto extractFacebookUserFromToken(TokenDto tokenDto) {
        UserDto userDto;
        try {
            Facebook facebook = new FacebookTemplate(tokenDto.getToken());
            String[] userFields = {"email", "first_name", "last_name", "birthday"};
            User user = facebook.fetchObject("me", User.class, userFields);

            userDto = UserDto.builder()
                    .firstName(user.getFirstName())
                    .lastName(user.getLastName())
                    .email(user.getEmail())
                    .birthDate(this.parseLocalDate(user.getBirthday(), "MM/dd/YYYY"))
                    .password(this.encoder.encode(this.defaultSocialPassword))
                    .build();
        } catch (Exception e) {
            throw new PetFinderException("Error: Impossible to connect with Facebook", e.getLocalizedMessage(), HttpStatus.BAD_REQUEST, null);
        }
        return userDto;
    }

    private ZonedDateTime parseLocalDate(String date, String datePattern) {
        if (StringUtils.isEmpty(date))
            return null;
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(datePattern);
        return ZonedDateTime.parse(date, formatter);
    }

    public LoginResponseDto connectWithGoogle(TokenDto tokenDto) {

        UserDto userDto = this.extractGoogleUserFromToken(tokenDto);

        if (!this.userRepository.existsByEmail(userDto.getEmail())) {
            userDto.setIsSocial(Boolean.TRUE);
            userDto = this.userService.createUser(userDto);
        } else {
            userDto = this.userService.getUserByEmail(userDto.getEmail());
        }

        return this.autoLoginUser(userDto);

    }

    private UserDto extractGoogleUserFromToken(TokenDto tokenDto) {
        UserDto userDto;
        try {
            NetHttpTransport httpTransport = new NetHttpTransport();
            JacksonFactory jacksonFactory = JacksonFactory.getDefaultInstance();
            GoogleIdTokenVerifier.Builder verifier = new GoogleIdTokenVerifier.Builder(httpTransport, jacksonFactory)
                    .setAudience(Arrays.asList(this.googleId));
            GoogleIdToken idToken = GoogleIdToken.parse(jacksonFactory, tokenDto.getToken());
            GoogleIdToken.Payload payload = idToken.getPayload();
            userDto = UserDto.builder().email(payload.getEmail())
                    .firstName((String) payload.get("given_name"))
                    .lastName((String) payload.get("family_name"))
                    .password(this.encoder.encode(this.defaultSocialPassword)).build();
        } catch (Exception e) {
            throw new PetFinderException("Error: Impossible to connect with Google", e.getLocalizedMessage(), HttpStatus.BAD_REQUEST, null);
        }
        return userDto;
    }
}
