package edu.uoc.petfinder.security;

import com.fasterxml.jackson.databind.ObjectMapper;
import edu.uoc.petfinder.dto.UserDto;
import edu.uoc.petfinder.exception.ApiExceptionHandler;
import edu.uoc.petfinder.exception.ErrorResponse;
import edu.uoc.petfinder.exception.PetFinderException;
import edu.uoc.petfinder.service.UserService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.util.StringUtils;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.time.ZonedDateTime;


@Slf4j
public class JwtTokenFilter extends OncePerRequestFilter {

    @Autowired
    private JwtTokenService tokenService;

    @Autowired
    private UserDetailsServiceImpl userDetailsService;

    @Autowired
    private ObjectMapper objectMapper;

    @Override
    protected void doFilterInternal(HttpServletRequest req, HttpServletResponse res, FilterChain chain) throws IOException, ServletException, PetFinderException {
        try {
            String jwtToken = parseJwt(req);
            if (jwtToken != null && this.tokenService.validateToken(jwtToken)) {
                this.retrieveAndAuthenticateUserFromToken(req, jwtToken);
            }
        } catch (PetFinderException ex) {
            this.sendErrorResponse(res, ex);
            return;
        }

        chain.doFilter(req, res);
    }

    private void sendErrorResponse(HttpServletResponse res, PetFinderException ex) throws IOException {
        res.setStatus(ex.getStatus().value());
        res.setContentType("application/json");
        PrintWriter out = res.getWriter();
        ErrorResponse errorResponse = ErrorResponse.builder()
                .message(ex.getMessage())
                .status(ex.getStatus())
                .debugMessage(ex.getDetails())
                .timestamp(ZonedDateTime.now())
                .code(ex.getCode())
                .build();
        out.print(this.objectMapper.writeValueAsString(errorResponse));
        out.flush();
    }

    private String retrieveAndAuthenticateUserFromToken(HttpServletRequest req, String jwtToken) {
        String email = this.tokenService.getEmailFromToken(jwtToken);

        UserDetails userDetails = userDetailsService.loadUserByUsername(email);
        UsernamePasswordAuthenticationToken authentication = new UsernamePasswordAuthenticationToken(userDetails, null,
                userDetails.getAuthorities());
        authentication.setDetails(new WebAuthenticationDetailsSource().buildDetails(req));

        SecurityContextHolder.getContext().setAuthentication(authentication);
        return email;
    }

    private String parseJwt(HttpServletRequest request) {
        String headerAuth = request.getHeader("Authorization");

        if (StringUtils.hasText(headerAuth) && headerAuth.startsWith("Bearer ")) {
            return headerAuth.substring(7, headerAuth.length());
        }

        return null;
    }
}
