package edu.uoc.petfinder.service.impl;

import com.jlefebure.spring.boot.minio.MinioException;
import com.jlefebure.spring.boot.minio.MinioService;
import edu.uoc.petfinder.dto.ImageDto;
import edu.uoc.petfinder.dto.PetDto;
import edu.uoc.petfinder.exception.PetFinderException;
import edu.uoc.petfinder.mapper.ApiMapper;
import edu.uoc.petfinder.model.Image;
import edu.uoc.petfinder.repository.ImageRepository;
import edu.uoc.petfinder.service.ImageCompressor;
import edu.uoc.petfinder.service.ImageService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Paths;
import java.time.Instant;
import java.util.Arrays;

@Service
@AllArgsConstructor
@Slf4j
public class ImageServiceImpl implements ImageService {

    private MinioService minioService;

    private ImageRepository imageRepository;

    private ImageCompressor imageCompressor;

    private ApiMapper mapper;

    @Override
    public PetDto removeImageFromPetDto(String imageId, PetDto petDto) {
        try {
            Image image = this.imageRepository.findById(imageId)
                    .orElseThrow(() -> new PetFinderException("Error: unknonwn image system file name", HttpStatus.NOT_FOUND));
            ImageDto imageDto = this.mapper.imageToImageDto(image);
            this.minioService.remove(Paths.get(imageDto.getSystemFileName()));
            petDto.removeImage(imageDto);
        } catch (MinioException e) {
            throw new PetFinderException("Error removing image: "+ imageId, HttpStatus.INTERNAL_SERVER_ERROR);
        }
        return petDto;
    }

    @Override
    public PetDto addImagesToPetDto(MultipartFile[] images, PetDto petDto) {
         Arrays.stream(images)
                .map(file -> this.buildImageFromFile(file))
                .map(image -> this.saveImage(image))
                .forEach(image -> petDto.addImage(image));

        return petDto;
    }

    private ImageDto buildImageFromFile(MultipartFile file) {
        try {
            return (ImageDto.builder()
                    .originalFileName(file.getOriginalFilename())
                    .fileResource(new InputStreamResource(file.getInputStream()))
                    .contentType(file.getContentType())
                    .contentLength(file.getSize())
                    .systemFileName(Instant.now().toEpochMilli() +"-"+file.getOriginalFilename()).build());
        } catch (IOException e) {
            throw new PetFinderException("Error building imageDTO: "+ file.getOriginalFilename(), HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Override
    public ImageDto fetchImage(String systemFileName, Boolean withFile) {
        ImageDto imageDto = null;
        try {
            imageDto = this.imageRepository.findBySystemFileName(systemFileName)
                    .orElseThrow(() -> new PetFinderException("Error: unknonwn image system file name", HttpStatus.NOT_FOUND));
            if (withFile) {
                InputStream inputStream = this.minioService.get(Paths.get(systemFileName));
                imageDto.setFileResource(new InputStreamResource(inputStream));
            }

        } catch (MinioException e) {
            log.error("Error fetching image: "+imageDto.toString(), e);
            throw new PetFinderException("Error fetching image to Minio", e, HttpStatus.INTERNAL_SERVER_ERROR, null);
        }
        return imageDto;
    }

    public ImageDto saveImage(ImageDto imageDto) {
        ImageDto createdImageDto = null;
        try {
            InputStream processedImage = this.imageCompressor.compressImage(imageDto, 0.6f, "2MB");
            this.minioService.upload(Paths.get(imageDto.getSystemFileName()),processedImage, imageDto.getContentType());
            Image savedImage = this.imageRepository.save(this.mapper.imageDtoToImage(imageDto));
            createdImageDto = this.mapper.imageToImageDto(savedImage);

        } catch (Exception e) {
            log.error("Error uploading image: "+imageDto.toString(), e);
            throw new PetFinderException("Error uploading image to Minio", e, HttpStatus.INTERNAL_SERVER_ERROR, null);
        }
        return createdImageDto;
    }
}
