package edu.uoc.petfinder.service.impl;

import edu.uoc.petfinder.constant.ActionType;
import edu.uoc.petfinder.dto.ActionDto;
import edu.uoc.petfinder.dto.PetDto;
import edu.uoc.petfinder.dto.SimplePetDto;
import edu.uoc.petfinder.dto.UserDto;
import edu.uoc.petfinder.exception.PetFinderException;
import edu.uoc.petfinder.mapper.ApiMapper;
import edu.uoc.petfinder.model.Pet;
import edu.uoc.petfinder.repository.ActionRepository;
import edu.uoc.petfinder.repository.PetRepository;
import edu.uoc.petfinder.service.ActionService;
import edu.uoc.petfinder.service.PetService;
import edu.uoc.petfinder.service.UserService;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.domain.*;

import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
public class PetServiceImpl implements PetService {

    private PetRepository petRepository;

    private UserService userService;

    private ActionRepository actionRepository;

    private ActionService actionService;

    private ApiMapper mapper;

    @Override
    @Transactional
    public PetDto createPet(PetDto petDto, String userEmail) {
        UserDto owner = this.userService.getUserByEmail(userEmail);
        petDto.setOwner(owner);
        ActionDto initialLostActionDto = this.fetchInitialAction(petDto);
        petDto = this.actionService.lost(initialLostActionDto, petDto, owner);
        Pet createdPet = this.petRepository.save(this.mapper.petDtoToPet(petDto));

        return this.mapper.petToPetDto(createdPet);
    }

    private ActionDto fetchInitialAction(PetDto petDto) {
        ActionDto initialLostActionDto;
        if (CollectionUtils.isEmpty(petDto.getActions())) {
            initialLostActionDto = this.buildInitialLostAction(petDto);
        } else {
            initialLostActionDto = petDto.getActions().get(0);
        }
        return initialLostActionDto;
    }

    private ActionDto buildInitialLostAction(PetDto petDto) {
        ActionDto initialLostActionDto = ActionDto.builder()
                .location(petDto.getLocation())
                .type(ActionType.LOST)
                .message("Missing")
                .build();
        return initialLostActionDto;
    }

    @Override
    public PetDto updatePet(PetDto petDto, String userEmail) {
        if (petDto.getId() == null) {
            return this.createPet(petDto, userEmail);
        } else {
            Pet updatedPet = this.petRepository.save(this.mapper.petDtoToPet(petDto));
            return this.mapper.petToPetDto(updatedPet);
        }
    }

    @Override
    public PetDto getPetById(String petId) {
        Pet pet = this.petRepository.findById(petId).orElseThrow(() -> new PetFinderException(String.format("Error: Pet with id %s not found", petId), HttpStatus.NOT_FOUND));
        return this.mapper.petToPetDto(pet);
    }

    @Override
    public PetDto getPetByIdAndOwner(String petId, UserDto owner) {
        Pet pet = this.petRepository.findByIdAndOwner(petId, this.mapper.userDtoToUser(owner))
                .orElseThrow(() -> new PetFinderException(String.format("Error: Pet with id %s not found", petId), HttpStatus.NOT_FOUND));
        return this.mapper.petToPetDto(pet);
    }

    @Override
    public PetDto mergeAndUpdatePet(PetDto previousPetDto, PetDto petToUpdate, UserDto userDto) {
        previousPetDto.setBirthDate(petToUpdate.getBirthDate());
        previousPetDto.setDescription(petToUpdate.getDescription());
        previousPetDto.setLocation(petToUpdate.getLocation());
        previousPetDto.setType(petToUpdate.getType());
        previousPetDto.setName(petToUpdate.getName());
        return this.updatePet(previousPetDto, userDto.getEmail());
    }

    @Override
    public Page<SimplePetDto> findAllPetsOrderedByDateDesc(Pageable pageable) {
        Pageable page = PageRequest.of(pageable.getPageNumber(), pageable.getPageSize(), Sort.Direction.DESC, "lastModifiedDate");
        Page<Pet> petsPage = this.petRepository.findPaginatedOrderByDate(page);
        return new PageImpl<>(petsPage
                .getContent()
                .stream()
                .map(this.mapper::petToSimplePetDto)
                .collect(Collectors.toList()),petsPage.getPageable(), petsPage.getTotalElements());

    }

    @Override
    public Page<SimplePetDto> findFoundPetsOrderedByDateDesc(Pageable pageable) {
        Pageable page = PageRequest.of(pageable.getPageNumber(), pageable.getPageSize(), Sort.Direction.DESC, "lastModifiedDate");
        Page<Pet> petsPage = this.petRepository.findFoundPetsPaginatedOrderByDate(page);
        return new PageImpl<>(petsPage
                .getContent()
                .stream()
                .map(this.mapper::petToSimplePetDto)
                .collect(Collectors.toList()),petsPage.getPageable(), petsPage.getTotalElements());

    }

    @Override
    public Page<SimplePetDto> searchPets(String search, Pageable pageable) {
        Page<Pet> petPage;
        if (StringUtils.isEmpty(search)) {
            petPage = this.petRepository.findAll(pageable);
        } else {
            Pet pet = new Pet();
            pet.setId(search);
            pet.setName(search);
            ExampleMatcher matcher = ExampleMatcher.matchingAny().withIgnoreNullValues().withStringMatcher(ExampleMatcher.StringMatcher.CONTAINING);
            petPage = this.petRepository.findAll(Example.of(pet, matcher), pageable);
        }
        List<SimplePetDto> simplePetDtos = petPage.getContent().stream().map(this.mapper::petToSimplePetDto).collect(Collectors.toList());
        return new PageImpl<>(simplePetDtos, petPage.getPageable(), petPage.getTotalElements());
    }

    @Override
    public void deletePet(String petId, Boolean isOwnPet, UserDto userDto) {
        Pet pet = this.petRepository.findById(petId).orElseThrow(() -> new PetFinderException("Error: deleting pet not found", HttpStatus.NOT_FOUND));
        if (isOwnPet && !StringUtils.equalsIgnoreCase(userDto.getEmail(), pet.getOwner().getEmail())) {
            throw new PetFinderException("Error: you are not allowed to delete the pet", HttpStatus.NOT_ACCEPTABLE);
        }
        pet.getActions().stream().forEach(this.actionRepository::delete);
        this.petRepository.delete(pet);
    }
}
