package edu.uoc.petfinder.service.impl;

import edu.uoc.petfinder.constant.RoleType;
import edu.uoc.petfinder.dto.SimplePetDto;
import edu.uoc.petfinder.dto.UserDto;
import edu.uoc.petfinder.exception.PetFinderException;
import edu.uoc.petfinder.mail.EmailData;
import edu.uoc.petfinder.mail.EmailFactory;
import edu.uoc.petfinder.mapper.ApiMapper;
import edu.uoc.petfinder.model.Role;
import edu.uoc.petfinder.model.User;
import edu.uoc.petfinder.repository.ActionRepository;
import edu.uoc.petfinder.repository.PetRepository;
import edu.uoc.petfinder.repository.RoleRepository;
import edu.uoc.petfinder.repository.UserRepository;
import edu.uoc.petfinder.service.UserService;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.domain.*;
import org.springframework.http.HttpStatus;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class UserServiceImpl implements UserService {

    private final UserRepository userRepository;

    private final RoleRepository roleRepository;

    private final PetRepository petRepository;

    private final ActionRepository actionRepository;

    private final ApiMapper mapper;

    private final PasswordEncoder passwordEncoder;

    private final EmailFactory emailFactory;

    @Override
    @Transactional
    public UserDto createUser(UserDto userDto) {
        if (this.userRepository.existsByEmail(userDto.getEmail())) {
            throw new PetFinderException("Error: that email already exists in our system", HttpStatus.NOT_ACCEPTABLE);
        }

        User user = this.mapper.userDtoToUser(userDto);
        user.setPassword(this.passwordEncoder.encode(userDto.getPassword()));

        Role userRole = this.roleRepository.findByName(RoleType.ROLE_USER).get();
        user.setRoles(Collections.singleton(userRole));

        user = this.userRepository.save(user);

        userDto = this.mapper.userToUserDto(user);

        this.emailFactory.accountCreatedMail(EmailData.builder().user(userDto).build());

        return userDto;
    }

    @Override
    public Boolean isEmailAvailable(String email) {
        return !this.userRepository.existsByEmail(email);
    }

    @Override
    public UserDto getUserByEmail(String email) {
        User user = this.userRepository.findByEmail(email).orElseThrow(() -> new PetFinderException("Error: Unable to find user", HttpStatus.BAD_REQUEST));
        return this.mapper.userToUserDto(user);
    }

    @Override
    public UserDto patchUserFields(UserDto connectedUser, UserDto updatedUserDto) {
        connectedUser.setBirthDate(updatedUserDto.getBirthDate());
        connectedUser.setFirstName(updatedUserDto.getFirstName());
        connectedUser.setLastName(updatedUserDto.getLastName());
        User savedUser = this.userRepository.save(this.mapper.userDtoToUser(connectedUser));
        return this.mapper.userToUserDto(savedUser);
    }

    @Override
    public Page<SimplePetDto> fetchPaginatedPetsByUser(UserDto connectedUser, Pageable pageable) {
        return this.petRepository.findPaginatedPetsByUserId(connectedUser.getId(), pageable);
    }

    @Override
    public Page<UserDto> searchUsers(String search, Pageable pageable) {
        Page<User> userPage;
        List<UserDto> userDtoList;
        if (StringUtils.isEmpty(search)) {
            userPage = this.userRepository.findAll(pageable);

        } else {
            userPage = this.searchUsersByExample(search, pageable);
        }
        userDtoList = userPage.getContent().stream().map(this.mapper::userToUserDto).collect(Collectors.toList());
        return new PageImpl<>(userDtoList, userPage.getPageable(), userPage.getTotalElements());
    }

    private Page<User> searchUsersByExample(String search, Pageable pageable) {
        Page<User> userPage;
        User user = new User();
        user.setEmail(search);
        user.setId(search);
        ExampleMatcher matcher = ExampleMatcher.matchingAny().withIgnoreNullValues().withStringMatcher(ExampleMatcher.StringMatcher.CONTAINING);
        userPage = this.userRepository.findAll(Example.of(user, matcher), pageable);
        return userPage;
    }

    @Override
    public void deleteUser(String userId) {
        User user = this.userRepository.findById(userId).orElseThrow(() -> new PetFinderException("Error: deleting user doesnt exist", HttpStatus.NOT_FOUND));
        this.petRepository.findAllByOwner(user).forEach(this.petRepository::delete);
        this.actionRepository.findAllByCreator(user).forEach(this.actionRepository::delete);
        this.userRepository.delete(user);
    }
}
