package edu.uoc.petfinder.controller;

import edu.uoc.petfinder.ApiControllerTest;
import edu.uoc.petfinder.constant.ActionType;
import edu.uoc.petfinder.dto.ActionDto;
import edu.uoc.petfinder.dto.PetDto;
import edu.uoc.petfinder.service.ActionService;
import edu.uoc.petfinder.service.PetService;
import edu.uoc.petfinder.service.UserService;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithAnonymousUser;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.security.test.context.support.WithSecurityContext;

import static org.hamcrest.Matchers.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;


class ActionControllerTest extends BaseControllerTest {

    @MockBean
    private ActionService actionService;

    @MockBean
    private PetService petService;

    @MockBean
    private UserService userService;


    @Test
    @WithMockUser(username = "email@email.es")
    void shouldNotAcceptForInvalidAction() throws Exception {
        //g
        ActionDto actionDto = ActionDto.builder().type(null).build();
        PetDto targetPet = new PetDto();

        String emailAsset = "email@email.com";
        super.mockAuthenticatedUserInjection(this.userService, emailAsset);

        //w
        super.mockMvc.perform(post("/pet/{petId}/lost", "ID123")
                .contentType(MediaType.APPLICATION_JSON)
                .content(super.objectMapper.writeValueAsString(actionDto))
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isBadRequest());

        super.mockMvc.perform(post("/pet/{petId}/found", "ID123")
                .contentType(MediaType.APPLICATION_JSON)
                .content(super.objectMapper.writeValueAsString(actionDto))
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isBadRequest());

        super.mockMvc.perform(post("/pet/{petId}/sighting", "ID123")
                .contentType(MediaType.APPLICATION_JSON)
                .content(super.objectMapper.writeValueAsString(actionDto))
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isBadRequest());

        //t
    }

    @Test
    @WithMockUser(username = "email@email.es")
    void shouldCreateLostAction() throws Exception {
        //g
        ActionDto actionDto = ActionDto.builder().type(ActionType.LOST).message("Lost message").build();
        PetDto targetPet = new PetDto();
        targetPet.setId("ID123");

        String emailAsset = "email@email.com";
        this.stubbingMethods(actionDto, targetPet, emailAsset);

        //w
        super.mockMvc.perform(post("/pet/{petId}/lost", "ID123")
                .contentType(MediaType.APPLICATION_JSON)
                .content(super.objectMapper.writeValueAsString(actionDto))
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.id", is(equalToIgnoringCase("ID123"))))
                .andExpect(jsonPath("$.actions", hasSize(1)))
                .andExpect(jsonPath("$.actions[0].type", is(ActionType.LOST.name())));
        //t
    }

    @Test
    @WithMockUser(username = "email@email.es")
    void shouldCreateSightingAction() throws Exception {
        //g
        ActionDto actionDto = ActionDto.builder().type(ActionType.SIGHTING).message("Sighting message").build();
        PetDto targetPet = new PetDto();
        targetPet.setId("ID123");

        String emailAsset = "email@email.com";
        this.stubbingMethods(actionDto, targetPet, emailAsset);

        //w
        super.mockMvc.perform(post("/pet/{petId}/sighting", "ID123")
                .contentType(MediaType.APPLICATION_JSON)
                .content(super.objectMapper.writeValueAsString(actionDto))
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.id", is(equalToIgnoringCase("ID123"))))
                .andExpect(jsonPath("$.actions", hasSize(1)))
                .andExpect(jsonPath("$.actions[0].type", is(ActionType.SIGHTING.name())));
        //t
    }

    @Test
    @WithMockUser(username = "email@email.es")
    void shouldCreateFoundAction() throws Exception {
        //g
        ActionDto actionDto = ActionDto.builder().type(ActionType.FOUND).message("Found message").build();
        PetDto targetPet = new PetDto();
        targetPet.setId("ID123");

        String emailAsset = "email@email.com";
        this.stubbingMethods(actionDto, targetPet, emailAsset);

        //w
        super.mockMvc.perform(post("/pet/{petId}/found", "ID123")
                .contentType(MediaType.APPLICATION_JSON)
                .content(super.objectMapper.writeValueAsString(actionDto))
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.id", is(equalToIgnoringCase("ID123"))))
                .andExpect(jsonPath("$.actions", hasSize(1)))
                .andExpect(jsonPath("$.actions[0].type", is(ActionType.FOUND.name())));
        //t
    }

    private void stubbingMethods(ActionDto actionDto, PetDto targetPet, String emailAsset) {
        super.mockAuthenticatedUserInjection(this.userService, emailAsset);

        when(this.petService.getPetById(anyString())).thenReturn(targetPet);
        targetPet.addAction(actionDto);
        when(this.actionService.sighting(any(), any(), any())).thenReturn(targetPet);
        when(this.petService.updatePet(any(), anyString())).thenReturn(targetPet);
    }
}