package edu.uoc.petfinder.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import edu.uoc.petfinder.constant.Genre;
import edu.uoc.petfinder.constant.PetType;
import edu.uoc.petfinder.dto.ImageDto;
import edu.uoc.petfinder.dto.LocationDto;
import edu.uoc.petfinder.dto.PetDto;
import edu.uoc.petfinder.dto.SimplePetDto;
import edu.uoc.petfinder.service.ImageService;
import edu.uoc.petfinder.service.PetService;
import edu.uoc.petfinder.service.UserService;
import edu.uoc.petfinder.spatial.SpatialService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.mockito.verification.VerificationMode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.geo.Distance;
import org.springframework.data.geo.GeoPage;
import org.springframework.data.geo.GeoResult;
import org.springframework.data.geo.GeoResults;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.util.UriComponentsBuilder;

import javax.servlet.ServletContext;

import java.util.Arrays;

import static org.hamcrest.Matchers.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.BDDMockito.given;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

class PetControllerTest extends BaseControllerTest  {

    @MockBean
    private PetService petService;

    @MockBean
    private UserService userService;

    @MockBean
    private ImageService imageService;

    @MockBean
    private SpatialService spatialService;

    @Test
    @WithMockUser(username = "email@email.es")
    void savePet() throws Exception {
        //G

        String emailAsset = "email@email.com";
        super.mockAuthenticatedUserInjection(this.userService, emailAsset);
        PetDto pet = buildDto();

        //W
        when(this.petService.createPet(any(), anyString())).thenReturn(pet);


        //T
        super.mockMvc.perform(post("/pet")
                .contentType(MediaType.APPLICATION_JSON)
                .content(super.objectMapper.writeValueAsString(pet))
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.id", is(equalToIgnoringCase("abc"))));
    }

    private PetDto buildDto() {
        PetDto pet = new PetDto();
        pet.setId("abc");
        pet.setName("Doggie");
        pet.setGenre(Genre.FEMALE);
        pet.setType(PetType.CAT);
        pet.setLocation(LocationDto.builder().latitude(45d).longitude(2d).build());
        return pet;
    }

    @Test
    void getFoundPetsWithLocation() throws Exception {
        //G
        SimplePetDto pet = new SimplePetDto();
        pet.setId("abc");
        pet.setName("Doggie");
        GeoPage<SimplePetDto> petDtoGeoPage = new GeoPage<>(new GeoResults<>(Arrays.asList(new GeoResult<>(pet, new Distance(23d)))));

        //W
        given(this.spatialService.searchFoundSimplePetByLocation(any(), any())).willReturn(petDtoGeoPage);

        //T
        MultiValueMap valueMap = new LinkedMultiValueMap<>();
        valueMap.add("latitude", "54879d");
        valueMap.add("longitude", "12478d");
        super.mockMvc.perform(MockMvcRequestBuilders.get("/pet/found")
                .queryParams(valueMap)
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.averageDistance").exists())
                .andExpect(jsonPath("$.content[0].content.id", is("abc")));

        verify(this.spatialService, times(1)).searchFoundSimplePetByLocation(any(),any());

    }

    @Test
    void getFoundPetsWithoutLocation() throws Exception {
        //G
        SimplePetDto pet = new SimplePetDto();
        pet.setId("abc");
        pet.setName("Doggie");
        Page<SimplePetDto> petDtos = new PageImpl<>(Arrays.asList(pet, pet));

        //W
        given(this.petService.findFoundPetsOrderedByDateDesc(any())).willReturn(petDtos);

        //T
        super.mockMvc.perform(MockMvcRequestBuilders.get("/pet/found")
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.averageDistance").doesNotExist())
                .andExpect(jsonPath("$.content.[0].id", is("abc")))
                .andExpect(jsonPath("$.content.[1].name", is("Doggie")))
                .andExpect(jsonPath("$.content", hasSize(2)));

        verify(this.petService, times(1)).findFoundPetsOrderedByDateDesc(any());

    }

    @Test
    @WithMockUser(username = "email@email.es")
    void deletePetById() throws Exception {
        //G
        String emailAsset = "email@email.com";
        super.mockAuthenticatedUserInjection(this.userService, emailAsset);

        //W
        //T
        super.mockMvc.perform(MockMvcRequestBuilders.delete("/pet/{petId}", "1234")
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isNoContent());

        verify(this.petService, times(1)).deletePet(any(), anyBoolean(), any());
    }

    @Test
    void getPetById() throws Exception {
        //G
        PetDto pet = buildDto();

        given(this.petService.getPetById(any())).willReturn(pet);
        //W
        //T
        super.mockMvc.perform(MockMvcRequestBuilders.get("/pet/{petId}", "1234")
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is("abc")))
                .andExpect(jsonPath("$.name", is("Doggie")));

        verify(this.petService, only()).getPetById(any());
    }

    @Test
    @WithMockUser(username = "email@email.es")
    void savePetImages() throws Exception {
        //G
        PetDto dto = this.buildDto();
        String emailAsset = "email@email.com";
        byte[] bytes = new String("abc").getBytes();
        MockMultipartFile file = new MockMultipartFile("text.jpg", "text.jpg", "image/jpg", bytes);
        super.mockAuthenticatedUserInjection(this.userService, emailAsset);
        ImageDto imageDto = ImageDto.builder().id("23DQSDezd3").build();

        //W
        given(this.petService.getPetByIdAndOwner(anyString(), any())).willReturn(dto);
        dto.addImage(imageDto);
        given(this.imageService.addImagesToPetDto(any(), any())).willReturn(dto);
        given(this.petService.updatePet(any(), anyString())).willReturn(dto);

        //T
        super.mockMvc.perform(MockMvcRequestBuilders.multipart("/pet/{petId}/images", "1234")
                .file(file)
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is("abc")))
                .andExpect(jsonPath("$.name", is("Doggie")))
                .andExpect(jsonPath("$.images", hasSize(1)));

        verify(this.petService, times(1)).getPetByIdAndOwner(anyString(), any());
        verify(this.imageService, times(1)).addImagesToPetDto(any(), any());
        verify(this.petService, times(1)).updatePet(any(), anyString());

    }

    @Test
    @WithMockUser(username = "email@email.es")
    void deleteImageFromPet() throws Exception {
        //G
        PetDto dto = this.buildDto();
        String emailAsset = "email@email.com";
        super.mockAuthenticatedUserInjection(this.userService, emailAsset);
        ImageDto imageDto = ImageDto.builder().id("23DQSDezd3").build();

        //W
        dto.addImage(imageDto);
        given(this.petService.getPetByIdAndOwner(anyString(), any())).willReturn(dto);
        dto.removeImage(imageDto);
        given(this.imageService.removeImageFromPetDto(any(), any())).willReturn(dto);
        given(this.petService.updatePet(any(), anyString())).willReturn(dto);

        //T
        super.mockMvc.perform(MockMvcRequestBuilders.delete("/pet/{petId}/images/{imageId}", "1234", "1234")
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is("abc")))
                .andExpect(jsonPath("$.name", is("Doggie")))
                .andExpect(jsonPath("$.images").doesNotExist());

        verify(this.petService, times(1)).getPetByIdAndOwner(anyString(), any());
        verify(this.imageService, times(1)).removeImageFromPetDto(any(), any());
        verify(this.petService, times(1)).updatePet(any(), anyString());


    }

    @Test
    @WithMockUser(username = "email@email.es")
    void updatePet() throws Exception {
        //G
        PetDto dto = this.buildDto();
        String emailAsset = "email@email.com";
        super.mockAuthenticatedUserInjection(this.userService, emailAsset);

        //W
        given(this.petService.mergeAndUpdatePet(any(), any(), any())).willReturn(dto);
        given(this.petService.getPetByIdAndOwner(any(), any())).willReturn(dto);

        //T

        super.mockMvc.perform(MockMvcRequestBuilders.put("/pet/{petId}", "1234")
                .content(super.objectMapper.writeValueAsString(dto))
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is("abc")))
                .andExpect(jsonPath("$.name", is("Doggie")));

        verify(this.petService, times(1)).mergeAndUpdatePet(any(), any(), any());
        verify(this.petService, times(1)).getPetByIdAndOwner(any(), any());
    }

    @Test
    void getPetsWithLocation() throws Exception {
        //G
        SimplePetDto pet = new SimplePetDto();
        pet.setId("abc");
        pet.setName("Doggie");
        GeoPage<SimplePetDto> petDtoGeoPage = new GeoPage<>(new GeoResults<>(Arrays.asList(new GeoResult<>(pet, new Distance(23d)))));

        //W
        given(this.spatialService.searchSimplePetByLocation(any(), any())).willReturn(petDtoGeoPage);

        //T
        MultiValueMap valueMap = new LinkedMultiValueMap<>();
        valueMap.add("latitude", "54879d");
        valueMap.add("longitude", "12478d");
        super.mockMvc.perform(MockMvcRequestBuilders.get("/pet")
                .queryParams(valueMap)
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.averageDistance").exists())
                .andExpect(jsonPath("$.content[0].content.id", is("abc")));

        verify(this.spatialService, times(1)).searchSimplePetByLocation(any(),any());
    }

    @Test
    void getPetsWithoutLocation() throws Exception {
        //G
        SimplePetDto pet = new SimplePetDto();
        pet.setId("abc");
        pet.setName("Doggie");
        Page<SimplePetDto> petDtos = new PageImpl<>(Arrays.asList(pet, pet));

        //W
        given(this.petService.findAllPetsOrderedByDateDesc(any())).willReturn(petDtos);

        //T
        super.mockMvc.perform(MockMvcRequestBuilders.get("/pet")
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.averageDistance").doesNotExist())
                .andExpect(jsonPath("$.content.[0].id", is("abc")))
                .andExpect(jsonPath("$.content.[1].name", is("Doggie")))
                .andExpect(jsonPath("$.content", hasSize(2)));

        verify(this.petService, times(1)).findAllPetsOrderedByDateDesc(any());
    }
}