package edu.uoc.petfinder.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import edu.uoc.petfinder.ApiControllerTest;
import edu.uoc.petfinder.dto.UserDto;
import edu.uoc.petfinder.mail.EmailService;
import edu.uoc.petfinder.security.WebSecurityConfig;
import edu.uoc.petfinder.service.UserService;
import org.apache.http.entity.ContentType;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.json.AutoConfigureJsonTesters;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockHttpServletResponse;
import org.springframework.security.test.context.support.WithAnonymousUser;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.TestPropertySource;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.util.MimeType;

import java.time.ZonedDateTime;

import static org.hamcrest.MatcherAssert.*;
import static org.hamcrest.Matchers.*;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

class UserControllerTest extends BaseControllerTest {

    @MockBean
    private UserService userService;

    @MockBean
    private EmailService emailService;


    @Test
    @WithAnonymousUser
    void userShouldNotFindItsOwnUser() throws Exception {
        //g
        //w
        MockHttpServletResponse response = mockMvc.perform(get("/api/user")
                .queryParam("email", "abc@abc.es")
                .contentType("application/json")
                .accept(MediaType.APPLICATION_JSON))
                .andReturn().getResponse();
        //t
        assertThat(response.getStatus(), is(404));
    }

    @Test
    @WithMockUser(username = "email@email.com")
    void registeredUserShouldGetItsOwnUser() throws Exception {
        //g
        String emailAsset = "email@email.com";
        super.mockAuthenticatedUserInjection(this.userService, emailAsset);

        //w
        this.mockMvc.perform(get("/user")
                .queryParam("email", emailAsset)
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.email", equalToIgnoringCase(emailAsset)));
        //t
    }

    @Test
    @WithMockUser(username = "email2@email.com")
    void shouldNotUpdateDifferentUser() throws Exception {
        //g
        String emailAsset = "email@email.com";
        super.mockAuthenticatedUserInjection(this.userService, emailAsset);

        //w
        this.mockMvc.perform(put("/user")
                .queryParam("email", emailAsset)
                .contentType("application/json")
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isBadRequest());
        //t
    }

    @Test
    @WithMockUser(username = "email@email.com")
    void updateOwnUserForOwnUser() throws Exception {
        //g
        String emailAsset = "email@email.com";
        super.mockAuthenticatedUserInjection(this.userService, emailAsset);

        UserDto updatedUserDto = UserDto.builder()
                .email(emailAsset)
                .firstName("Ricard")
                .lastName("Molina")
                .birthDate(ZonedDateTime.now().minusDays(5L))
                .build();

        when(this.userService.patchUserFields(any(), any())).thenReturn(updatedUserDto);

        //w
        this.mockMvc.perform(put("/user").queryParam("email", emailAsset)
                .contentType("application/json")
                .content(super.objectMapper.writeValueAsString(updatedUserDto))
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.email", equalToIgnoringCase(emailAsset)))
                .andExpect(jsonPath("$.firstName", equalToIgnoringCase("Ricard")))
                .andExpect(jsonPath("$.lastName", is("Molina")));
        //t
        verify(this.userService, times(1)).patchUserFields(any(), any());
    }
}