package edu.uoc.petfinder.service.impl;

import edu.uoc.petfinder.constant.ActionType;
import edu.uoc.petfinder.constant.Genre;
import edu.uoc.petfinder.constant.PetType;
import edu.uoc.petfinder.controller.BaseControllerTest;
import edu.uoc.petfinder.dto.ActionDto;
import edu.uoc.petfinder.dto.LocationDto;
import edu.uoc.petfinder.dto.PetDto;
import edu.uoc.petfinder.dto.UserDto;
import edu.uoc.petfinder.exception.PetFinderException;
import edu.uoc.petfinder.mail.EmailFactory;
import edu.uoc.petfinder.mapper.ApiMapper;
import edu.uoc.petfinder.model.Action;
import edu.uoc.petfinder.repository.ActionRepository;
import edu.uoc.petfinder.state.*;
import io.minio.MinioClient;
import org.hamcrest.Matchers;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentMatchers;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.BDDMockito.given;
import static org.mockito.Mockito.*;

@ExtendWith(SpringExtension.class)
@SpringBootTest
@ActiveProfiles("test")
class ActionServiceImplTest {

    @MockBean
    private ActionRepository actionRepository;

    @Autowired
    private ApiMapper mapper;

    @MockBean
    private ActionContext actionContext;

    @MockBean
    private StateFactory stateFactory;

    @MockBean
    private EmailFactory emailFactory;

    @Autowired
    private ActionServiceImpl actionService;

    @MockBean
    private JavaMailSender javaMailSender;

    @MockBean
    private MinioClient minioClient;

    @Test
    void saveAction() {
        //G
        ActionDto actionDto = ActionDto.builder().id("43534tr3").type(ActionType.LOST).message("Lorem").build();
        Action action = this.mapper.actionDtoToAction(actionDto);

        //W
        given(this.actionRepository.save(ArgumentMatchers.any())).willReturn(action);

        //T
        ActionDto resultAction = this.actionService.saveAction(actionDto);
        assertEquals(actionDto, resultAction);

        verify(this.actionRepository, times(1)).save(ArgumentMatchers.any());
    }

    @Test
    void lostWithIncorrectAction() {
        //G
        ActionDto actionDto = ActionDto.builder().id("43534tr3").type(ActionType.FOUND).message("Lorem").build();
        PetDto petDto = this.buildPetDto();
        UserDto dto = UserDto.builder().email("abc@abc.es").password("abc").build();

        //W
        PetFinderException exception = assertThrows(PetFinderException.class, () -> this.actionService.lost(actionDto, petDto, dto));
    }

    @Test
    void lost() {
        //G
        ActionDto actionDto = ActionDto.builder().id("43534tr3").type(ActionType.LOST).message("Lorem").build();
        PetDto petDto = this.buildPetDto();
        petDto.addAction(actionDto);
        UserDto dto = UserDto.builder().email("abc@abc.es").password("abc").build();

        //W

        LostState state = Mockito.mock(LostState.class);
        given(this.actionContext.getState()).willReturn(state);
        given(state.lost(ArgumentMatchers.any())).willReturn(petDto);

        //T
        PetDto resultPet = this.actionService.lost(actionDto, petDto, dto);
        assertThat(resultPet.getId(), is("abc"));
        assertThat(resultPet.getActions(), hasSize(1));
        assertThat(resultPet.getActions().get(0).getType(), is(ActionType.LOST));

        verify(state, times(1)).lost(any());

    }

    private PetDto buildPetDto() {
        PetDto pet = new PetDto();
        pet.setId("abc");
        pet.setName("Doggie");
        pet.setGenre(Genre.FEMALE);
        pet.setType(PetType.CAT);
        pet.setLocation(LocationDto.builder().latitude(45d).longitude(2d).build());
        return pet;
    }

    @Test
    void foundWithIncorrectAction() {
        //G
        ActionDto actionDto = ActionDto.builder().id("43534tr3").type(ActionType.LOST).message("Lorem").build();
        PetDto petDto = this.buildPetDto();
        UserDto dto = UserDto.builder().email("abc@abc.es").password("abc").build();

        //W
        PetFinderException exception = assertThrows(PetFinderException.class, () -> this.actionService.found(actionDto, petDto, dto));
    }

    @Test
    void found() {
        //G
        ActionDto actionDto = ActionDto.builder().id("43534tr3").type(ActionType.FOUND).message("Lorem").build();
        PetDto petDto = this.buildPetDto();
        petDto.addAction(actionDto);
        UserDto dto = UserDto.builder().email("abc@abc.es").password("abc").build();

        //W
        FoundState state = Mockito.mock(FoundState.class);
        given(this.actionContext.getState()).willReturn(state);
        given(state.found(ArgumentMatchers.any())).willReturn(petDto);

        //T
        PetDto resultPet = this.actionService.found(actionDto, petDto, dto);
        assertThat(resultPet.getId(), is("abc"));
        assertThat(resultPet.getActions(), hasSize(1));
        assertThat(resultPet.getActions().get(0).getType(), is(ActionType.FOUND));

        verify(state, times(1)).found(any());
    }

    @Test
    void sightingWithIncorrectAction() {
        //G
        ActionDto actionDto = ActionDto.builder().id("43534tr3").type(ActionType.LOST).message("Lorem").build();
        PetDto petDto = this.buildPetDto();
        UserDto dto = UserDto.builder().email("abc@abc.es").password("abc").build();

        //W
        PetFinderException exception = assertThrows(PetFinderException.class, () -> this.actionService.sighting(actionDto, petDto, dto));
    }

    @Test
    void sighting() {
        //G
        ActionDto actionDto = ActionDto.builder().id("43534tr3").type(ActionType.SIGHTING).message("Lorem").build();
        PetDto petDto = this.buildPetDto();
        petDto.addAction(actionDto);
        UserDto dto = UserDto.builder().email("abc@abc.es").password("abc").build();

        //W
        SightingState state = Mockito.mock(SightingState.class);
        given(this.actionContext.getState()).willReturn(state);
        given(state.sighting(ArgumentMatchers.any())).willReturn(petDto);

        //T
        PetDto resultPet = this.actionService.sighting(actionDto, petDto, dto);
        assertThat(resultPet.getId(), is("abc"));
        assertThat(resultPet.getActions(), hasSize(1));
        assertThat(resultPet.getActions().get(0).getType(), is(ActionType.SIGHTING));

        verify(state, times(1)).sighting(any());
    }
}