import { ChangeDetectionStrategy, Component, EventEmitter, OnInit, Output } from '@angular/core';
import { AbstractControl, AsyncValidator, AsyncValidatorFn, FormBuilder, FormControl, FormGroup, Validators } from '@angular/forms';
import { of } from 'rxjs';
import { debounceTime, distinctUntilChanged, map, switchMap, tap } from 'rxjs/operators';
import { AuthService } from 'src/app/services/auth.service';
import User from 'src/app/models/User';

@Component({
  selector: 'app-register-form',
  templateUrl: './register-form.component.html',
  styleUrls: ['./register-form.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class RegisterFormComponent implements OnInit {

  public registerForm: FormGroup

  @Output()
  public register = new EventEmitter();

  constructor(private formBuilder: FormBuilder, private authService: AuthService) { }

  ngOnInit(): void {
    this.registerForm = this.formBuilder.group({
      firstName: this.formBuilder.control('', [Validators.required]),
      lastName: this.formBuilder.control('', [Validators.required]),
      email: this.formBuilder.control('', [Validators.required, Validators.email], [this.validateEmailAvailability(this.authService)]),
      password: this.formBuilder.control('', [Validators.required, Validators.pattern('^(?=.*[a-z])(?=.*[A-Z])(?=.*\\d).{8,}$')])
    })
  }

  submitRegisterForm() {
    const loginUser: User = this.registerForm.value;
    this.register.emit(loginUser);
  }

  ngOnDestroy() {
    this.registerForm.reset();
  }

  get firstNameInput(): FormControl {
    return this.registerForm.get('firstName') as FormControl;
  }

  get lastNameInput(): FormControl {
    return this.registerForm.get('lastName') as FormControl;
  }

  get emailInput(): FormControl {
    return this.registerForm.get('email') as FormControl;
  }

  get passwordInput(): FormControl {
    return this.registerForm.get('password') as FormControl;
  }

  validateEmailAvailability(authService: AuthService): AsyncValidatorFn {
    return (control: AbstractControl) => {
      return of(control.value).pipe(
        distinctUntilChanged(),
        debounceTime(500),
        switchMap((email: string) => this.authService.isEmailAvailable(email).pipe(
          tap(data => console.log(data)),
          map(({isEmailAvailable}) => (isEmailAvailable ? null : { emailNotAvailable: true }))
        ))
      )
    }
  }

}
