import { AfterViewInit, ViewChild } from '@angular/core';
import { Component, OnInit } from '@angular/core';
import { FormBuilder } from '@angular/forms';
import { LeafletDirective } from '@asymmetrik/ngx-leaflet';
import { latLng, Layer, LeafletMouseEvent, MapOptions, marker, Marker, tileLayer } from 'leaflet';
import { BsModalRef } from 'ngx-bootstrap/modal';
import { takeUntil, take } from 'rxjs/operators';
import { ActionType } from 'src/app/models/ActionType';
import Pet from 'src/app/models/Pet';
import { BaseComponent } from 'src/app/shared/base.component';
import { environment } from 'src/environments/environment';
import { ActionsService } from '../actions.service';
import { Location } from '../../../models/Location';
import { NotificationService } from 'src/app/services/notification.service';
import { PetService } from '../../pet/pet.service';
import { sortBy } from 'lodash';
import { TranslateService } from '@ngx-translate/core';
import LeafletFactory from 'src/app/shared/LeafletFactory';

@Component({
  selector: 'app-actions-modal',
  templateUrl: './actions-modal.component.html',
  styleUrls: ['./actions-modal.component.scss'],
})
export class ActionsModalComponent
  extends BaseComponent
  implements OnInit, AfterViewInit {
  pet: Pet;

  actionType: ActionType;

  spinnerVisible = false;

  message: string;

  options: MapOptions;

  location: Location;

  actionLocation: Location;

  locationLayer: Layer;

  @ViewChild(LeafletDirective)
  map: LeafletDirective;

  constructor(
    public actionsModal: BsModalRef,
    private actionsService: ActionsService,
    private notificationService: NotificationService,
    private petService: PetService,
    private translate: TranslateService
  ) {
    super();
  }

  ngOnInit(): void {
    console.log(this.actionsModal);
    this.actionsService.pet$
      .pipe(takeUntil(this.destroy$))
      .subscribe(({ pet, action, location }) => {
        this.pet = pet;
        this.actionType = action;
        this.location = location || pet.location;
      });
    this.options = {
      layers: [tileLayer(environment.maps_provider, { maxZoom: 18 })],
      zoom: 13,
      center: latLng(
        this.location.lat,
        this.location.long
      ),
    };
  }

  ngAfterViewInit() {
    setTimeout(() => {
      this.map.getMap().invalidateSize();
    }, 100);
  }

  submitAction(event) {
    this.spinnerVisible = true;
    this.actionsService.createActionInPet(this.pet.id, this.actionType, { message: this.message, location: this.actionLocation, type: this.actionType}).pipe(take(1))
    .subscribe(() => {
      this.actionsModal.hide();
      this.notificationService.success(this.translate.instant('response.ok.addedAction'));
      this.petService.reload();
    }, () => this.notificationService.error(this.translate.instant('response.ko.addedAction')), () => this.spinnerVisible = false);
  }

  close() {
    this.actionsModal.hide();
  }

  onMapClick(event: LeafletMouseEvent) {
    const {lat, lng} = event.latlng;
    this.actionLocation = {lat, long: lng};
    const mark: Layer = LeafletFactory.marker(event.latlng);
    this.locationLayer ? this.map.getMap().removeLayer(this.locationLayer) : null;
    this.locationLayer = mark; 
    this.map.getMap().addLayer(this.locationLayer);
  }

  buildAcceptanceBtnMessage() {
    switch (this.actionType) {
      case ActionType.SIGHTING:
        return this.translate.instant('actionModal.yes-sighting');

      case ActionType.FOUND:
        return this.translate.instant('actionModal.yes-found');

      case ActionType.LOST:
        return this.translate.instant('actionModal.yes-lost');

      default:
        throw new Error('Unknown action type');
    }
  }

  buildTitle() {
    switch (this.actionType) {
      case ActionType.SIGHTING:
        return this.translate.instant('actionModal.sighting-title', {name: this.pet.name});

      case ActionType.FOUND:
        return this.translate.instant('actionModal.found-title', {name: this.pet.name});

      case ActionType.LOST:
        return this.translate.instant('actionModal.lost-title', {name: this.pet.name});

      default:
        throw new Error('Unknown action type');
    }
  }
}
