import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { BehaviorSubject, Observable, zip } from 'rxjs';
import { distinctUntilChanged, switchMap, tap, takeUntil, filter, debounceTime } from 'rxjs/operators';
import GeoContent from 'src/app/models/GeoContent';
import Page from 'src/app/models/Page';
import Pet from 'src/app/models/Pet';
import { GeolocationService } from 'src/app/services/geolocation.service';
import { ScrollEvent, ScrollService } from 'src/app/services/scroll.service';
import { BaseComponent } from 'src/app/shared/base.component';
import { PetContentType } from '../home/home.component';
import { PetService } from '../pet/pet.service';
import { Location } from '../../models/Location';
import { NotificationService } from 'src/app/services/notification.service';
import { NgxMasonryOptions } from 'ngx-masonry';
import { MASONRY_OPTS } from 'src/app/shared/masonry-opts';
import { TranslateService } from '@ngx-translate/core';
import { LayoutService } from '../layout/layout.service';

@Component({
  selector: 'app-found',
  templateUrl: './found.component.html',
  styleUrls: ['./found.component.scss'],
})
export class FoundComponent extends BaseComponent implements OnInit {
  isLoading = false;
  empty = false;
  pets: GeoContent<Pet>[] | Pet[] = [];
  contentType: PetContentType;
  private page$ = new BehaviorSubject(0);
  last = false;
  opts: NgxMasonryOptions = MASONRY_OPTS;

  constructor(
    private scrollService: ScrollService,
    private notificationService: NotificationService,
    public petService: PetService,
    private router: Router,
    private layoutService: LayoutService,
    private translate: TranslateService,
    public geolocationService: GeolocationService
  ) {
    super();
  }

  ngOnInit(): void {
    this.page$.next(0);
    this.layoutService.setPageTitle(this.translate.instant('pageTitle.found'));
    this.fetchInitialFoundPets();
    this.fetchOnScroll();
  }

  goToDetail(id: string) {
    this.router.navigate(['/pet', id]);
  }

  private fetchInitialFoundPets() {
    this.fetchPetsFromApi().subscribe(data => {
      this.pets = data.content;
    }, (err) => {
      this.isLoading = false;
      this.notificationService.error( this.translate.instant('response.ko.loadingPets'))
    });
  }

  private fetchPetsFromApi(): Observable<any> {
    this.isLoading = true;
    return zip(this.geolocationService.location, this.page$).pipe(
      distinctUntilChanged(),
      switchMap(([loc, page]) => (!!loc ?
        this.petService.fetchFoundGeolocationPet((loc as Location), { page }) :
        this.petService.fetchFoundRecentPetsPet({ page }))),
      tap(({last, empty}: Page<any>) => {
        this.last = last;
        this.empty = empty;
      }),
      tap(() => this.isLoading = false),
      tap(data => !!data.averageDistance ? this.contentType = PetContentType.GEOCONTENT : PetContentType.CONTENT),
      takeUntil(this.destroy$)
    );
  }

  private fetchOnScroll() {
    this.scrollService.scroll$.pipe(
      filter((event: ScrollEvent) => event.percent > 80 && !this.last),
      debounceTime(500),
      tap(() => this.page$.next(this.page$.value + 1)),
      switchMap(() => (this.fetchPetsFromApi()))
    ).subscribe((data: any) =>  {
      data.content?.forEach(c => this.pets.push(c));
    },
    () => {
      this.isLoading = false;
      this.notificationService.error( this.translate.instant('response.ko.loadingPets'))
    });
  }

  get gridTitle() {
    return this.contentType === PetContentType.GEOCONTENT ? this.translate.instant('found.geo-title') : this.translate.instant('found.title');
  }
}
