import { AfterViewInit, Component, HostListener, OnDestroy, OnInit, Renderer2, ViewChild } from '@angular/core';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { NgxMasonryComponent, NgxMasonryOptions } from 'ngx-masonry';
import { BehaviorSubject, empty, Observable, zip } from 'rxjs';
import { combineAll, debounceTime, distinctUntilChanged, filter, map, switchMap, take, takeUntil, tap } from 'rxjs/operators';
import GeoContent from 'src/app/models/GeoContent';
import { Location } from 'src/app/models/Location';
import Page from 'src/app/models/Page';
import Pet from 'src/app/models/Pet';
import { AuthService } from 'src/app/services/auth.service';
import { GeolocationService } from 'src/app/services/geolocation.service';
import { NotificationService } from 'src/app/services/notification.service';
import { ScrollEvent, ScrollService } from 'src/app/services/scroll.service';
import { UserService } from 'src/app/services/user.service';
import { BaseComponent } from 'src/app/shared/base.component';
import { MASONRY_OPTS } from 'src/app/shared/masonry-opts';
import { AccessModalService } from '../access/access-modal.service';
import { LayoutService } from '../layout/layout.service';
import { CreatePetModalService } from '../pet/create-pet-modal.service';
import { PetService } from '../pet/pet.service';

export enum PetContentType {
  GEOCONTENT ='GECONTENT', CONTENT ='CONTENT'
}

@Component({
  selector: 'app-home',
  templateUrl: './home.component.html',
  styleUrls: ['./home.component.scss'],
})
export class HomeComponent extends BaseComponent implements OnInit, AfterViewInit {

  isLoading = false;

  empty = false;

  pets: GeoContent<Pet>[] | Pet[] = [];

  contentType: PetContentType;

  opts: NgxMasonryOptions = MASONRY_OPTS;

  @ViewChild(NgxMasonryComponent)
  masonryGrid: NgxMasonryComponent;

  private page$ = new BehaviorSubject(0);
  last = false;

  constructor(
    public geolocationService: GeolocationService,
    public userService: UserService,
    public accessModalService: AccessModalService,
    public petService: PetService,
    private layoutService: LayoutService,
    private createPetModalService: CreatePetModalService,
    private scrollService: ScrollService,
    private notificationService: NotificationService,
    private translate: TranslateService,
    private router: Router
  ) {
    super();
  }

  ngOnInit(): void {
    this.page$.next(0);
    this.fetchInitialPets();
    this.fetchOnScroll();
    this.listenToReload();
  }

  ngAfterViewInit() {
    this.layoutService.setPageTitle(null);
  }

  private listenToReload() {
    this.petService.reload$.pipe(takeUntil(this.destroy$)).subscribe(() => {
      window.scrollTo(0,0);
      this.pets = [];
      this.masonryGrid.reloadItems();
      this.page$.next(0);
      this.fetchInitialPets();
    });
  }

  goToDetail(id: string) {
    this.router.navigate(['/pet', id]);
  }

  private fetchOnScroll() {
    this.scrollService.scroll$.pipe(
      takeUntil(this.destroy$),
      filter((event: ScrollEvent) => event.percent > 80 && !this.last),
      debounceTime(500),
      tap(() => this.page$.next(this.page$.value + 1)),
      switchMap(() => (this.fetchPetsFromApi()))
    ).subscribe((data: any) =>  {
      data.content?.forEach(c => this.pets.push(c));
      this.masonryGrid.reloadItems();
    },
    (err) => {
      this.isLoading = false;
      this.notificationService.error( this.translate.instant('response.ko.loadingPets'))
    }, () => this.isLoading = false);
  }

  private fetchInitialPets() {
    this.fetchPetsFromApi().subscribe(data => {
      this.pets = data.content;
    }, (err) => {
      this.isLoading = false;
      this.notificationService.error( this.translate.instant('response.ko.loadingPets'))
    }, () => this.isLoading = false);
  }


  private fetchPetsFromApi(): Observable<any> {
    this.isLoading = true;
    return zip(this.geolocationService.location, this.page$).pipe(
      distinctUntilChanged(),
      switchMap(([loc, page]) => (!!loc ?
        this.petService.fetchGeolocationPet((loc as Location), { page }) :
        this.petService.fetchRecentPetsPet({ page }))),
      tap(({last, empty}: Page<any>) => {
        this.last = last;
        this.empty = empty;
      }),
      tap(() => this.isLoading = false),
      tap(data => !!data.averageDistance ? this.contentType = PetContentType.GEOCONTENT : PetContentType.CONTENT),
      take(1)
    );
  }

  private canCreatePet(): Observable<boolean> {
    return zip(
      this.geolocationService.location,
      this.userService.isAuthenticated
    ).pipe(
      map(([hasLocation, isAuth]) => (hasLocation && isAuth ? true : false)),
      tap((canCreatePet) =>
        !canCreatePet ? this.accessModalService.redirectToAccesModal() : null
      ),
      takeUntil(this.destroy$)
    );
  }

  createPet() {
    this.canCreatePet().subscribe((canCreate) => {
      if (canCreate) {
        this.createPetModalService.openModal()
      }
    });
  }

  get gridTitle() {
    return this.contentType === PetContentType.GEOCONTENT ? this.translate.instant('home.geo-title') : this.translate.instant('home.title');
  }
}
