import { AfterViewInit, Component, OnInit, ViewChild } from '@angular/core';
import {
  FormBuilder,
  FormControl,
  FormGroup,
  Validators,
} from '@angular/forms';
import { PetType } from 'src/app/models/PetType';
import {
  NgFileSelectDirective,
  UploaderOptions,
  UploadFile,
} from 'ngx-uploader';
import { BaseComponent } from 'src/app/shared/base.component';
import Pet from 'src/app/models/Pet';
import { PetService } from '../pet.service';
import { switchMap, take } from 'rxjs/operators';
import { NotificationService } from 'src/app/services/notification.service';
import { BsModalRef } from 'ngx-bootstrap/modal';
import { Genre } from 'src/app/models/Genre';
import { TranslateService } from '@ngx-translate/core';

@Component({
  selector: 'app-create-pet-modal',
  templateUrl: './create-pet-modal.component.html',
  styleUrls: ['./create-pet-modal.component.scss'],
})
export class CreatePetModalComponent extends BaseComponent implements OnInit {
  public uploaderOpts: UploaderOptions = {
    allowedContentTypes: ['image/jpeg', 'image/png'],
    concurrency: 5,
    maxUploads: 5,
    maxFileSize: 4000000, // 4MB
  };

  PetType = PetType;
  Genre = Genre;

  imageSelectorError = false;
  spinnerVisible = false;
  creationError = false;
  createPetForm: FormGroup;

  @ViewChild(NgFileSelectDirective)
  imageSelector: NgFileSelectDirective;

  files: UploadFile[] = [];

  constructor(
    private fb: FormBuilder,
    private translate: TranslateService,
    private petService: PetService,
    private notificationService: NotificationService,
    public createPetModal: BsModalRef
  ) {
    super();
  }

  ngOnInit(): void {
    this.createPetForm = this.fb.group({
      name: this.fb.control('', [Validators.required]),
      type: this.fb.control(PetType.DOG, [Validators.required]),
      genre: this.fb.control(Genre.FEMALE, [Validators.required]),
      description: this.fb.control('', [Validators.maxLength(150)]),
      birthDate: this.fb.control(''),
      lostMessage: this.fb.control('', [Validators.maxLength(150)]),
    });
  }

  get name(): FormControl {
    return this.createPetForm.get(['name']) as FormControl;
  }

  get type(): FormControl {
    return this.createPetForm.get(['type']) as FormControl;
  }

  get genre(): FormControl {
    return this.createPetForm.get(['genre']) as FormControl;
  }

  get description(): FormControl {
    return this.createPetForm.get(['description']) as FormControl;
  }

  get lostMessage(): FormControl {
    return this.createPetForm.get(['lostMessage']) as FormControl;
  }

  submitCreateForm() {
    this.initRequest();
    const {
      name,
      type,
      description,
      birthDate,
      lostMessage,
      genre,
    }: any = this.createPetForm.value;
    const pet: Pet = { name, type, description, birthDate, genre };
    this.petService
      .createLostPet(pet, lostMessage)
      .pipe(
        switchMap((createdPet: Pet) =>
          this.petService.addImagesToPet(createdPet, this.files)
        )
      )
      .subscribe(
        (pet: Pet) => {
          this.completeRequest(false);
          this.resetImagesInput();
          this.createPetForm.reset();
          this.petService.reload();
          this.notificationService.success(
            this.translate.instant('response.ok.addPetMessage', {
              name: pet.name,
            }),
            this.translate.instant('response.ok.addPetTitle')
          );
          this.createPetModal.hide();
          console.log();
        },
        () => this.completeRequest(true)
      );
  }

  onUploadOutput(event) {
    if (event.type === 'rejected') {
      this.imageSelectorError = true;
    }
    if (event.type === 'addedToQueue') {
      this.files.push(event.file);
    }
  }

  initRequest() {
    this.creationError = false;
    this.spinnerVisible = true;
  }
  completeRequest(isError = false) {
    this.creationError = isError;
    this.spinnerVisible = true;
  }

  resetImagesInput() {
    this.files = [];
    this.imageSelector.upload.queue = [];
    this.imageSelectorError = false;
  }
}
