import { AfterViewInit, Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import {
  LeafletDirective,
  LeafletDirectiveWrapper,
} from '@asymmetrik/ngx-leaflet';
import { TranslateService } from '@ngx-translate/core';
import {
  tileLayer,
  latLng,
  circle,
  MapOptions,
  Path,
  LatLngBounds,
  latLngBounds,
} from 'leaflet';
import { orderBy, sortBy } from 'lodash';
import { filter, map, switchMap, takeUntil } from 'rxjs/operators';
import { Action } from 'src/app/models/Action';
import { ActionType } from 'src/app/models/ActionType';
import { Image } from 'src/app/models/Image';
import Pet from 'src/app/models/Pet';
import { PetType } from 'src/app/models/PetType';
import User from 'src/app/models/User';
import { AnalyticsService } from 'src/app/services/analytics.service';
import { UserService } from 'src/app/services/user.service';
import { BaseComponent } from 'src/app/shared/base.component';
import { environment } from 'src/environments/environment';
import { ActionsModalService } from '../../actions/actions-modal.service';
import { LayoutService } from '../../layout/layout.service';
import { PetService } from '../pet.service';
import { TitleGenerator } from './title-generator';

@Component({
  selector: 'app-pet-detail',
  templateUrl: './pet-detail.component.html',
  styleUrls: ['./pet-detail.component.scss'],
})
export class PetDetailComponent extends BaseComponent implements OnInit {

  ActionType = ActionType;

  pet: Pet;

  today = new Date();

  options: MapOptions;

  fitBounds: LatLngBounds;

  layers: Path[];

  @ViewChild(LeafletDirective)
  map: LeafletDirective;

  canEdit = false;

  constructor(
    private layoutService: LayoutService,
    private route: ActivatedRoute,
    private router: Router,
    private userService: UserService,
    private petService: PetService,
    private analytics: AnalyticsService,
    private titleGenerator: TitleGenerator,
    private actionsModalService: ActionsModalService,
    public translate: TranslateService
  ) {
    super();
  }

  ngOnInit(): void {
    this.layoutService.setExtendedHeader(false);
    this.pet = this.route.snapshot.data.pet;
    this.layoutService.setPageTitle(this.translate.instant('pageTitle.petDetail', {name: this.pet.name}));
    this.analytics.viewItem(this.pet);
    this.generateMapData();
    this.listenForReload();
    this.canEditPet();
  }

  private canEditPet() {
    this.userService.user.pipe(
      filter(user => !!user),
      map((user: User) => this.pet.owner.email === user.email ? true : false),
      takeUntil(this.destroy$)
    ).subscribe(canEdit => this.canEdit = canEdit);
  }

  private listenForReload() {
    this.petService.reload$.pipe(
      takeUntil(this.destroy$),
      switchMap(() => this.petService.getPetDetail(this.pet.id))
    ).subscribe(pet => this.pet = pet);
  }

  private generateMapData() {
    this.options = {
      layers: [tileLayer(environment.maps_provider, { maxZoom: 18 })],
      zoom: 13,
      center: latLng(
        this.pet.lastState.location.lat,
        this.pet.lastState.location.long
      ),
    };

    this.layers = [
      circle(
        [this.pet.lastState.location.lat, this.pet.lastState.location.long],
        { radius: 1000, color: 'red', stroke: false }
      ),
    ];
  }

  buildImageUrl(image: Image): string {
    return `${environment.api_endpoint}/image/${image.systemFileName}`;
  }

  buildCaptionPhrase() {
    return this.titleGenerator.generateTitle(this.pet);
  }

  goToEdit() {
    this.router.navigate(['/user', 'pet', this.pet.id, 'edit'])
  }

  displayActionDetails(action: Action) {
    this.actionsModalService.openDetailActionModal(action);
  }


  sortActions(pet: Pet) {
    if (!pet || !pet.actions) {
      return [];
    }
    return orderBy(pet.actions, ['createdDate'], ['desc'])
  }
}
