import { Injectable } from '@angular/core';
import { UploadFile } from 'ngx-uploader';
import { iif, Observable, of, Subject } from 'rxjs';
import { filter, map, switchMap, tap } from 'rxjs/operators';
import { Action } from 'src/app/models/Action';
import { ActionType } from 'src/app/models/ActionType';
import GeoContent from 'src/app/models/GeoContent';
import { Location } from 'src/app/models/Location';
import Page from 'src/app/models/Page';
import { PageParams } from 'src/app/models/PageParams';
import Pet from 'src/app/models/Pet';
import {Image} from 'src/app/models/Image';
import { GeolocationService } from 'src/app/services/geolocation.service';
import { RestService } from 'src/app/services/rest.service';

@Injectable({
  providedIn: 'root'
})
export class PetService {

  private static DEFAULT_PAGE_SIZE = 10;

  private reloadSbjct = new Subject();

  constructor(private geolocationService: GeolocationService, private rest: RestService) {}

  public createLostPet(pet: Pet, lostMessage?: string) {
    return this.geolocationService.location.pipe(
      filter(location => !!location),
      map((location: Location) => {
        const action: Action = { message: lostMessage, location, type: ActionType.LOST };
        !pet.actions ? pet.actions = [] : null;
        pet.actions.push(action);
        pet.location = location;
        return pet
      }),
      switchMap((pet: Pet) => this.createPet(pet))
    )
  }

  public addImagesToPet(createdPet: Pet, files: UploadFile[] = [] ): Observable<Pet> {
    let filesFormData: FormData = null;
    if (files.length) {
      filesFormData = new FormData();
      files.map(f => f.nativeFile).forEach(file => filesFormData.append(`images`, file));
      return this.createImagesForPet(createdPet.id, filesFormData);
    }
    return of(createdPet);

  }

  public deletePetImage(pet: Pet, image: Image): Observable<Pet> {
    const url = `/pet/${pet.id}/images/${image.id}`;
    return this.rest.delete(url);
  }

  public reload() {
    this.reloadSbjct.next(true);
  }

  get reload$() {
    return this.reloadSbjct.asObservable();
  }

  private createImagesForPet(petId: string, formData: FormData): Observable<any> {
    return this.rest.post(`/pet/${petId}/images`, formData);
  }

  private createPet(pet: Pet): Observable<Pet> {
    return this.rest.post('/pet', pet);
  }

  public updatePet(pet: Pet): Observable<Pet> {
    return this.rest.put(`/pet/${pet.id}`, pet);
  }

  public fetchGeolocationPet(loc: Location, page: PageParams): Observable<Page<GeoContent<Pet>>> {
    const url = `/pet?page=${page.page}&size=${page.size | PetService.DEFAULT_PAGE_SIZE}&latitude=${loc.lat}&longitude=${loc.long}`;
    return this.rest.get(url);
  }

  public fetchRecentPetsPet(page: PageParams): Observable<Page<Pet>> {
    const url = `/pet?page=${page.page}&size=${page.size | PetService.DEFAULT_PAGE_SIZE}`;
    return this.rest.get(url);
  }

  public fetchFoundGeolocationPet(loc: Location, page: PageParams): Observable<Page<GeoContent<Pet>>> {
    const url = `/pet/found?page=${page.page}&size=${page.size | PetService.DEFAULT_PAGE_SIZE}&latitude=${loc.lat}&longitude=${loc.long}`;
    return this.rest.get(url);
  }

  public fetchFoundRecentPetsPet(page: PageParams): Observable<Page<Pet>> {
    const url = `/pet/found?page=${page.page}&size=${page.size | PetService.DEFAULT_PAGE_SIZE}`;
    return this.rest.get(url);
  }

  public getPetDetail(id: string): Observable<Pet> {
    return this.rest.get(`/pet/${id}`);
  }

  public fetchOwnPets(page: PageParams): Observable<Page<Pet>> {
    const url = `/user/pet?page=${page.page}&size=${page.size | PetService.DEFAULT_PAGE_SIZE}`;
    return this.rest.get(url);
  }

  public searchPets(search = '', page: PageParams = { page: 0}): Observable<Page<Pet>> {
    return this.rest.get(`/admin/pets?search=${search}&page=${page.page}&size=5`);
  }

  public deletePet(pet: Pet) {
    return this.rest.delete(`/admin/pet/${pet.id}`);
  }

  public deleteUserPet(pet: Pet) {
    return this.rest.delete(`/pet/${pet.id}`);
  }
}
