import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { take, tap } from 'rxjs/operators';
import User from '../models/User';
import { RestService } from './rest.service';
import { UserService } from './user.service';
import jwt_decode from "jwt-decode";
import { AnalyticsService, LoginType } from './analytics.service';

@Injectable({
  providedIn: 'root'
})
export class AuthService {

  private tokenSbjct = new BehaviorSubject<String>(this.retrieveTokenOnInit());

  constructor(private rest: RestService, private userService: UserService, private analytics: AnalyticsService) { }

  set token(token: string) {
    localStorage.setItem('token', token);
    this.tokenSbjct.next(token);
  }

  clearToken() {
    localStorage.removeItem('token')
    this.tokenSbjct.next(null);
  }

  logout() {
    this.clearToken();
    this.userService.clearUser();
  }

  login(user: User): Observable<any> {
    return this.rest.post('/auth/signin', user).pipe(
      tap(({token}) => this.token = token),
      tap(() => this.analytics.login(LoginType.LOCAL)),
      tap(({firstName, lastName, email, roles}) => this.userService.setUser({firstName, lastName, email, roles})),
    );
  }

  signup(user: User): Observable<any> {
    return this.rest.post('/auth/signup', user).pipe(
      tap(({token}) => this.token = token),
      tap(() => this.analytics.signup()),
      tap(({firstName, lastName, email, roles}) => this.userService.setUser({firstName, lastName, email, roles})),
    );
  }

  facebookLogin(token: string): Observable<any> {
    return this.rest.post('/auth/facebook', {token}).pipe(
      tap(({token}) => this.token = token),
      tap(() => this.analytics.login(LoginType.FACEBOOK)),
      tap(({firstName, lastName, email, roles}) => this.userService.setUser({firstName, lastName, email, roles})),
    );
  }

  googleLogin(token: string): Observable<any> {
    return this.rest.post('/auth/google', {token}).pipe(
      tap(({token}) => this.token = token),
      tap(() => this.analytics.login(LoginType.GOOGLE)),
      tap(({firstName, lastName, email, roles}) => this.userService.setUser({firstName, lastName, email, roles})),
    );
  }

  isEmailAvailable(email: string): Observable<any> {
    return this.rest.get(`/user/available?email=${email}`);
  }

  private retrieveTokenOnInit() {
    const token = localStorage.getItem('token');
    if (!token) {
      return null;
    }
    const tokenDeccrypt: any = jwt_decode(token);
    const expDate = new Date(tokenDeccrypt.exp * 1000);
    const now = new Date();
    if (expDate <= now) {
      console.error('Token expired : '+token);
      localStorage.removeItem('token')
      this.userService.clearUser();
      return null;
    }
    return token;


  }


  get token$() {
    return this.tokenSbjct.asObservable();
  }
}
